# -*- coding: utf-8 -*-
import time

import autoit as auto
from main.decorator import *

TIMEOUT = 30


class GAutoItMap:
    WinWait = 'WinWait'
    WinWaitByHandle = 'WinWaitByHandle'
    WinClose = 'WinClose'
    WinCloseByHandle = 'WinCloseByHandle'
    WinKill = 'WinKill'
    Send = 'Send'
    WinKillByHandle = 'WinKillByHandle'
    WinActivate = 'WinActivate'
    WinActivateByHandle = 'WinActivateByHandle'
    WinWaitActivate = 'WinWaitActivate'
    WinWaitActivateByHandle = 'WinWaitActivateByHandle'
    WinExists = 'WinExists'
    WinExistsByHandle = 'WinExistsByHandle'
    WinActive = 'WinActive'
    WinActiveByHandle = 'WinActiveByHandle'
    WinGetCaretPos = 'WinGetCaretPos'
    WinGetClassList = 'WinGetClassList'
    WinGetClassListByHandle = 'WinGetClassListByHandle'
    WinGetClientSize = 'WinGetClientSize'
    WinGetClientSizeByHandle = 'WinGetClientSizeByHandle'
    WinGetHandle = 'WinGetHandle'
    WinGetHandleAsText = 'WinGetHandleAsText'
    WinGetPos = 'WinGetPos'
    WinGetPosByHandle = 'WinGetPosByHandle'
    WinGetProcess = 'WinGetProcess'
    WinGetProcessByHandle = 'WinGetProcessByHandle'
    WinGetState = 'WinGetState'
    WinGetStateByHandle = 'WinGetStateByHandle'
    WinGetText = 'WinGetText'
    WinGetTextByHandle = 'WinGetTextByHandle'
    WinGetTitle = 'WinGetTitle'
    WinGetTitleByHandle = 'WinGetTitleByHandle'
    WinMenuSelectItem = 'WinMenuSelectItem'
    WinMenuSelectItemByHandle = 'WinMenuSelectItemByHandle'
    WinMinimizeAll = 'WinMinimizeAll'
    WinMinimizeAllUndo = 'WinMinimizeAllUndo'
    WinMove = 'WinMove'
    WinMoveByHandle = 'WinMoveByHandle'
    WinTopMost = 'WinTopMost'
    WinUnTop = 'WinUnTop'
    WinTopMostByHandle = 'WinTopMostByHandle'
    WinUnTopByHandle = 'WinUnTopByHandle'
    WinHide = 'WinHide'
    WinHideByHandle = 'WinHideByHandle'
    WinShow = 'WinShow'
    WinShowByHandle = 'WinShowByHandle'
    WinMin = 'WinMin'
    WinMinByHandle = 'WinMinByHandle'
    WinMax = 'WinMax'
    WinMaxByHandle = 'WinMaxByHandle'
    WinRestore = 'WinRestore'
    WinRestoreByHandle = 'WinRestoreByHandle'
    WinNormal = 'WinNormal'
    WinNormalByHandle = 'WinNormalByHandle'
    WinDisable = 'WinDisable'
    WinDisableByHandle = 'WinDisableByHandle'
    WinMinDisable = 'WinMinDisable'
    WinMinDisableByHandle = 'WinMinDisableByHandle'
    WinNa = 'WinNa'
    WinNaByHandle = 'WinNaByHandle'
    WinDefault = 'WinDefault'
    WinDefaultByHandle = 'WinDefaultByHandle'
    WinSetTile = 'WinSetTile'
    WinSetTitleByHandle = 'WinSetTitleByHandle'
    WinSetTrans = 'WinSetTrans'
    WinSetTransByHandle = 'WinSetTransByHandle'
    WinWaitClose = 'WinWaitClose'
    WinWaitCloseByHandle = 'WinWaitCloseByHandle'
    WinWaitNotActive = 'WinWaitNotActive'
    WinWaitNotActiveByHandle = 'WinWaitNotActiveByHandle'
    CtlLeftClick = 'CtlLeftClick'
    CtlLeftClickByHandle = 'CtlLeftClickByHandle'
    CtlDoubleclick = 'CtlDoubleclick'
    CtlDoubleClickByHandle = 'CtlDoubleClickByHandle'
    CtlRightClick = 'CtlRightClick'
    CtlRightClickByHandle = 'CtlRightClickByHandle'
    CtlRightDoubleClick = 'CtlRightDoubleClick'
    CtlRightDoubleClickByHandle = 'CtlRightDoubleClickByHandle'
    CtlSend = 'CtlSend'
    CtlSendByHandle = 'CtlSendByHandle'
    CtlSetText = 'CtlSetText'
    CtlSetTextByHandle = 'CtlSetTextByHandle'
    CtlCommand = 'CtlCommand'
    CtlCommandByHandle = 'CtlCommandByHandle'
    CtlListView = 'CtlListView'
    CtlListViewByHandle = 'CtlListViewByHandle'
    CtlDisable = 'CtlDisable'
    CtlDisableByHandle = 'CtlDisableByHandle'
    CtlEnable = 'CtlEnable'
    CtlEnableByHandle = 'CtlEnableByHandle'
    CtlFocus = 'CtlFocus'
    CtlFocusByHandle = 'CtlFocusByHandle'
    CtlGetFocus = 'CtlGetFocus'
    CtlGetFocusByHandle = 'CtlGetFocusByHandle'
    CtlGetHandle = 'CtlGetHandle'
    CtlGetHandleAsText = 'CtlGetHandleAsText'
    CtlGetPos = 'CtlGetPos'
    CtlGetPosByHandle = 'CtlGetPosByHandle'
    CtlGetText = 'CtlGetText'
    CtlGetTextByHandle = 'CtlGetTextByHandle'
    CtlHide = 'CtlHide'
    CtlHideByHandle = 'CtlHideByHandle'
    CtlMove = 'CtlMove'
    CtlMoveByHandle = 'CtlMoveByHandle'
    CtlShow = 'CtlShow'
    CtlShowByHandle = 'CtlShowByHandle'
    CtlTreeView = 'CtlTreeView'
    CtlTreeViewByHandle = 'CtlTreeViewByHandle'


class GAutoItFieldsMap:
    openApp = 'openApp'
    winTitle = 'winTitle'
    winHandle = 'winHandle'
    winText = 'winText'
    ctlHandle = 'ctlHandle'
    ctlClassNameNN = 'ctlClassNameNN'
    treeViewCommand = 'treeViewCommand'
    treeViewExtra1 = 'treeViewExtra1'
    treeViewExtra2 = 'treeViewExtra2'
    ctlMoveX = 'ctlMoveX'
    ctlMoveY = 'ctlMoveY'
    ctlMoveWidth = 'ctlMoveWidth'
    ctlMoveHeight = 'ctlMoveHeight'
    listViewCommand = 'listViewCommand'
    listViewExtra1 = 'listViewExtra1'
    listViewExtra2 = 'listViewExtra2'
    command = 'command'
    commandExtra = 'commandExtra'
    ctlText = 'ctlText'
    menuItem = 'menuItem'
    winMoveX = 'winMoveX'
    winMoveY = 'winMoveY'
    winMoveHeight = 'winMoveHeight'
    winMoveWeight = 'winMoveWeight'
    winNewTitle = 'winNewTitle'
    winTrans = 'winTrans'
    operation = 'operation'
    ctlSendValue = 'ctlSendValue'
    sendValue = 'sendValue'
    raiseIfNotExist = 'raiseIfNotExist'


class GAutoIt:
    def __init__(self, **kwargs):
        self.kwargs = kwargs.get('operation_detail')

    def start(self):
        self.OpenAPP()
        switch = self.__switch_middle()
        for operation in self.Operation:
            fuc = switch.get(operation, None)
            # print(fuc)
            if fuc:
                fuc()
            else:
                pass

    def OpenAPP(self):
        if self.OpenApp == '':
            pass
        else:
            # print(self.OpenApp)
            auto.run(self.OpenApp)

    def __switch_middle(self):
        switch = dict()
        # 控件操作
        switch[GAutoItMap.CtlLeftClick] = self.CtlLeftClick
        switch[GAutoItMap.CtlLeftClickByHandle] = self.CtlLeftClickByHandle
        switch[GAutoItMap.CtlRightClick] = self.CtlRightClick
        switch[GAutoItMap.CtlRightClickByHandle] = self.CtlRightClickByHandle
        switch[GAutoItMap.CtlDoubleclick] = self.CtlDoubleclick
        switch[GAutoItMap.CtlDoubleClickByHandle] = self.CtlDoubleClickByHandle
        switch[GAutoItMap.CtlRightDoubleClick] = self.CtlRightDoubleClick
        switch[GAutoItMap.CtlRightDoubleClickByHandle] = self.CtlRightDoubleClickByHandle
        switch[GAutoItMap.CtlSend] = self.CtlSend
        switch[GAutoItMap.CtlSendByHandle] = self.CtlSendByHandle
        switch[GAutoItMap.CtlSetText] = self.CtlSetText
        switch[GAutoItMap.CtlSetTextByHandle] = self.CtlSetTextByHandle
        switch[GAutoItMap.CtlShow] = self.CtlShow
        switch[GAutoItMap.CtlShowByHandle] = self.CtlShowByHandle
        switch[GAutoItMap.CtlTreeView] = self.CtlTreeView
        switch[GAutoItMap.CtlTreeViewByHandle] = self.CtlTreeViewByHandle
        switch[GAutoItMap.CtlCommand] = self.CtlCommand
        switch[GAutoItMap.CtlCommandByHandle] = self.CtlCommandByHandle
        switch[GAutoItMap.CtlDisable] = self.CtlDisable
        switch[GAutoItMap.CtlDisableByHandle] = self.CtlDisableByHandle
        switch[GAutoItMap.CtlEnable] = self.CtlEnable
        switch[GAutoItMap.CtlEnableByHandle] = self.CtlEnableByHandle
        switch[GAutoItMap.CtlFocus] = self.CtlFocus
        switch[GAutoItMap.CtlFocusByHandle] = self.CtlFocusByHandle
        switch[GAutoItMap.CtlGetFocus] = self.CtlGetFocus
        switch[GAutoItMap.CtlGetFocusByHandle] = self.CtlGetFocusByHandle
        switch[GAutoItMap.CtlGetHandle] = self.CtlGetHandle
        switch[GAutoItMap.CtlGetHandleAsText] = self.CtlGetHandleAsText
        switch[GAutoItMap.CtlGetPos] = self.CtlGetPos
        switch[GAutoItMap.CtlGetPosByHandle] = self.CtlGetPosByHandle
        switch[GAutoItMap.CtlGetText] = self.CtlGetText
        switch[GAutoItMap.CtlGetTextByHandle] = self.CtlGetTextByHandle
        switch[GAutoItMap.CtlHide] = self.CtlHide
        switch[GAutoItMap.CtlHideByHandle] = self.CtlHideByHandle
        switch[GAutoItMap.CtlSend] = self.CtlSend
        switch[GAutoItMap.CtlListView] = self.CtlListView
        switch[GAutoItMap.CtlListViewByHandle] = self.CtlListViewByHandle
        switch[GAutoItMap.CtlMove] = self.CtlMove
        switch[GAutoItMap.CtlMoveByHandle] = self.CtlMoveByHandle
        # 窗口操作
        switch[GAutoItMap.WinClose] = self.WinClose
        switch[GAutoItMap.WinCloseByHandle] = self.WinCloseByHandle
        switch[GAutoItMap.WinActivate] = self.WinActivate
        switch[GAutoItMap.WinActivateByHandle] = self.WinActivateByHandle
        switch[GAutoItMap.WinExists] = self.WinExists
        switch[GAutoItMap.WinExistsByHandle] = self.WinExistsByHandle
        switch[GAutoItMap.WinWait] = self.WinWait
        switch[GAutoItMap.WinWaitByHandle] = self.WinWaitByHandle
        switch[GAutoItMap.WinWaitActivate] = self.WinWaitActivate
        switch[GAutoItMap.WinWaitActivateByHandle] = self.WinWaitActivateByHandle
        switch[GAutoItMap.WinWaitNotActive] = self.WinWaitNotActive
        switch[GAutoItMap.WinWaitNotActiveByHandle] = self.WinWaitNotActiveByHandle
        switch[GAutoItMap.WinActive] = self.WinActive
        switch[GAutoItMap.WinActiveByHandle] = self.WinActiveByHandle
        switch[GAutoItMap.WinWaitClose] = self.WinWaitClose
        switch[GAutoItMap.WinWaitCloseByHandle] = self.WinWaitCloseByHandle
        switch[GAutoItMap.WinGetCaretPos] = self.WinGetCaretPos
        switch[GAutoItMap.WinGetClassList] = self.WinGetClassList
        switch[GAutoItMap.WinGetClassListByHandle] = self.WinGetClassListByHandle
        switch[GAutoItMap.WinGetClientSize] = self.WinGetClientSize
        switch[GAutoItMap.WinGetClientSizeByHandle] = self.WinGetClientSizeByHandle
        switch[GAutoItMap.WinGetHandle] = self.WinGetHandle
        switch[GAutoItMap.WinGetHandleAsText] = self.WinGetHandleAsText
        switch[GAutoItMap.WinGetProcess] = self.WinGetProcess
        switch[GAutoItMap.WinGetProcessByHandle] = self.WinGetProcessByHandle
        switch[GAutoItMap.WinGetState] = self.WinGetState
        switch[GAutoItMap.WinGetStateByHandle] = self.WinGetStateByHandle
        switch[GAutoItMap.WinGetText] = self.WinGetText
        switch[GAutoItMap.WinGetTextByHandle] = self.WinGetTextByHandle
        switch[GAutoItMap.WinGetPos] = self.WinGetPos
        switch[GAutoItMap.WinGetPosByHandle] = self.WinGetPosByHandle
        switch[GAutoItMap.WinGetTitle] = self.WinGetTitle
        switch[GAutoItMap.WinGetTitleByHandle] = self.WinGetTitleByHandle
        switch[GAutoItMap.WinSetTile] = self.WinSetTile
        switch[GAutoItMap.WinSetTitleByHandle] = self.WinSetTitleByHandle
        switch[GAutoItMap.WinSetTrans] = self.WinSetTrans
        switch[GAutoItMap.WinSetTransByHandle] = self.WinSetTransByHandle
        switch[GAutoItMap.WinTopMost] = self.WinTopMost
        switch[GAutoItMap.WinTopMostByHandle] = self.WinTopMostByHandle
        switch[GAutoItMap.WinUnTop] = self.WinUnTop
        switch[GAutoItMap.WinUnTopByHandle] = self.WinUnTopByHandle
        switch[GAutoItMap.WinMin] = self.WinMin
        switch[GAutoItMap.WinMinByHandle] = self.WinMinByHandle
        switch[GAutoItMap.WinMax] = self.WinMax
        switch[GAutoItMap.WinMaxByHandle] = self.WinMaxByHandle
        switch[GAutoItMap.WinMenuSelectItem] = self.WinMenuSelectItem
        switch[GAutoItMap.WinMenuSelectItemByHandle] = self.WinMenuSelectItemByHandle
        switch[GAutoItMap.WinMinDisable] = self.WinMinDisable
        switch[GAutoItMap.WinMinDisableByHandle] = self.WinMinDisableByHandle
        switch[GAutoItMap.WinMinimizeAll] = self.WinMinimizeAll
        switch[GAutoItMap.WinMinimizeAllUndo] = self.WinMinimizeAllUndo
        switch[GAutoItMap.WinMove] = self.WinMove
        switch[GAutoItMap.WinMoveByHandle] = self.WinMoveByHandle
        switch[GAutoItMap.WinNa] = self.WinNa
        switch[GAutoItMap.WinNaByHandle] = self.WinNaByHandle
        switch[GAutoItMap.WinNormal] = self.WinNormal
        switch[GAutoItMap.WinNormalByHandle] = self.WinNormalByHandle
        switch[GAutoItMap.WinRestore] = self.WinRestore
        switch[GAutoItMap.WinRestoreByHandle] = self.WinRestoreByHandle
        switch[GAutoItMap.WinHide] = self.WinHide
        switch[GAutoItMap.WinHideByHandle] = self.WinHideByHandle
        switch[GAutoItMap.WinShow] = self.WinShow
        switch[GAutoItMap.WinShowByHandle] = self.WinShowByHandle
        switch[GAutoItMap.WinDefault] = self.WinDefault
        switch[GAutoItMap.WinDefaultByHandle] = self.WinDefaultByHandle
        switch[GAutoItMap.WinDisable] = self.WinDisable
        switch[GAutoItMap.WinDisableByHandle] = self.WinDisableByHandle
        switch[GAutoItMap.WinKill] = self.WinKill
        switch[GAutoItMap.WinKillByHandle] = self.WinKillByHandle
        switch[GAutoItMap.Send] = self.Send
        # 鼠标操作
        # process 操作
        return switch

    def __fields_middle(self):
        fields = dict()
        fields[GAutoItFieldsMap.openApp] = self.OpenApp
        fields[GAutoItFieldsMap.winTitle] = self.WinTitle
        fields[GAutoItFieldsMap.winText] = self.WinText
        fields[GAutoItFieldsMap.winHandle] = self.WinHandle
        fields[GAutoItFieldsMap.ctlHandle] = self.CtlHandle
        fields[GAutoItFieldsMap.ctlClassNameNN] = self.CtlClassNameNN
        fields[GAutoItFieldsMap.operation] = self.Operation
        fields[GAutoItFieldsMap.sendValue] = self.SendValue
        fields[GAutoItFieldsMap.ctlSendValue] = self.CtlSendValue
        # 控件操作 可选字段
        fields[GAutoItFieldsMap.treeViewCommand] = self.TreeViewCommand
        fields[GAutoItFieldsMap.treeViewExtra1] = self.TreeViewExtra1
        fields[GAutoItFieldsMap.treeViewExtra2] = self.TreeViewExtra2
        fields[GAutoItFieldsMap.ctlMoveX] = self.CtlMoveX
        fields[GAutoItFieldsMap.ctlMoveY] = self.CtlMoveY
        fields[GAutoItFieldsMap.ctlMoveHeight] = self.CtlMoveHeight
        fields[GAutoItFieldsMap.ctlMoveWidth] = self.CtlMoveWidth
        fields[GAutoItFieldsMap.listViewCommand] = self.ListViewCommand
        fields[GAutoItFieldsMap.listViewExtra1] = self.ListViewExtra1
        fields[GAutoItFieldsMap.listViewExtra2] = self.ListViewExtra2
        fields[GAutoItFieldsMap.command] = self.Command
        fields[GAutoItFieldsMap.commandExtra] = self.Command
        fields[GAutoItFieldsMap.ctlText] = self.CtlText

        # 窗口操作 可选字段
        fields[GAutoItFieldsMap.menuItem] = self.MenuItem
        fields[GAutoItFieldsMap.winMoveHeight] = self.WinMoveHeight
        fields[GAutoItFieldsMap.winMoveWeight] = self.WinMoveWeight
        fields[GAutoItFieldsMap.winMoveX] = self.WinMoveX
        fields[GAutoItFieldsMap.winMoveY] = self.WinMoveY
        fields[GAutoItFieldsMap.winNewTitle] = self.WinNewTitle
        fields[GAutoItFieldsMap.winTrans] = self.WinTrans
        fields[GAutoItFieldsMap.raiseIfNotExist] = self.raiseIfNotExist
        return fields

    def get_field(self):
        return self.__fields_middle()

    @property
    @TypeChecking(str)
    def OpenApp(self):
        app = self.kwargs.get(GAutoItFieldsMap.openApp, '')
        return app

    @property
    @TypeChecking(str)
    def WinTitle(self) -> str:
        win_title = self.kwargs.get(GAutoItFieldsMap.winTitle, '')
        return win_title

    # @property
    # def WinClass(self):
    #     win_class = self.kwargs.get('WinClass', '')
    #     return win_class

    @property
    @TypeChecking(int)
    def WinHandle(self):
        win_handle = self.kwargs.get(GAutoItFieldsMap.winHandle, '')
        if win_handle == '':
            return None
        return win_handle

    @property
    def WinText(self):
        win_text = self.kwargs.get(GAutoItFieldsMap.winText, '')
        return win_text

    # @property
    # def CtlClass(self):
    #     ctl_class = self.kwargs.get('CtlClass', '')
    #     return ctl_class

    @property
    @TypeChecking(int)
    def CtlHandle(self):
        ctl_handle = self.kwargs.get(GAutoItFieldsMap.ctlHandle, '')
        if ctl_handle == '':
            return None
        else:
            return ctl_handle

    # @property
    # def CtlText(self):
    #     ctl_text = self.kwargs.get('CtlText', '')
    #     return ctl_text

    def raiseIfNotExist(self):
        value = self.kwargs.get(GAutoItFieldsMap.ctlHandle, True)
        if value == '':
            return value
        return value

    @property
    def CtlClassNameNN(self):
        ctl_class_name_nn = self.kwargs.get(GAutoItFieldsMap.ctlClassNameNN, '')
        if ctl_class_name_nn == '':
            return None
        return ctl_class_name_nn

    @property
    def Operation(self):
        operation = self.kwargs.get(GAutoItFieldsMap.operation, '')
        if operation != '':
            operation = operation.split(',')
            return operation
        else:
            return []

    @property
    def SendValue(self):
        value = self.kwargs.get('SendValue', '')
        return value

    @property
    def CtlSendValue(self):
        value = self.kwargs.get(GAutoItFieldsMap.ctlSendValue, '')
        return value

    @property
    def TreeViewCommand(self):
        tree_view_command = self.kwargs.get(GAutoItFieldsMap.treeViewCommand, '')
        if tree_view_command == '':
            return None
        return tree_view_command

    @property
    def TreeViewExtra1(self):
        tree_view_extra1 = self.kwargs.get(GAutoItFieldsMap.treeViewExtra1, '')
        return tree_view_extra1

    @property
    def TreeViewExtra2(self):
        tree_view_extra2 = self.kwargs.get(GAutoItFieldsMap.treeViewExtra2, '')
        return tree_view_extra2

    @property
    # @TypeChecking(int)
    def CtlMoveX(self):
        ctl_move_x = self.kwargs.get(GAutoItFieldsMap.ctlMoveX, '')
        if ctl_move_x == '':
            return None
        return ctl_move_x

    @property
    # @TypeChecking(int)
    def CtlMoveY(self):
        ctl_move_y = self.kwargs.get(GAutoItFieldsMap.ctlMoveY, '')
        if ctl_move_y == '':
            return None
        return ctl_move_y

    @property
    @TypeChecking(int)
    def CtlMoveWidth(self):
        ctl_move_width = self.kwargs.get(GAutoItFieldsMap.ctlMoveWidth, -1)
        return int(ctl_move_width)

    @property
    # @TypeChecking(int)
    def CtlMoveHeight(self):
        ctl_move_height = self.kwargs.get(GAutoItFieldsMap.ctlMoveHeight, -1)
        return int(ctl_move_height)

    @property
    def ListViewCommand(self):
        list_view_command = self.kwargs.get(GAutoItFieldsMap.listViewCommand, '')
        if list_view_command == '':
            return None
        return list_view_command

    @property
    def ListViewExtra1(self):
        list_view_extra1 = self.kwargs.get(GAutoItFieldsMap.listViewExtra1, '')
        # if list_view_extra1 == '':
        #     return None
        return list_view_extra1

    @property
    def ListViewExtra2(self):
        list_view_extra2 = self.kwargs.get(GAutoItFieldsMap.listViewExtra2, '')
        # if list_view_extra2 == '':
        #     return None
        return list_view_extra2

    @property
    def Command(self):
        command = self.kwargs.get(GAutoItFieldsMap.command, '')
        if command == '':
            return None
        return command

    @property
    def CommandExtra(self):
        command_extra = self.kwargs.get(GAutoItFieldsMap.commandExtra, '')
        return command_extra

    @property
    def CtlText(self):
        ctl_text = self.kwargs.get(GAutoItFieldsMap.ctlText, '')
        return ctl_text

    @property
    def MenuItem(self):
        menu_item = self.kwargs.get(GAutoItFieldsMap.menuItem, '')
        return menu_item

    @property
    # @TypeChecking(int)
    def WinMoveX(self):
        move_x = self.kwargs.get(GAutoItFieldsMap.winMoveX, '')
        if move_x == '':
            return None
        return move_x

    @property
    # @TypeChecking(int)
    def WinMoveY(self):
        move_y = self.kwargs.get(GAutoItFieldsMap.winMoveY, '')
        if move_y == '':
            return None
        return move_y

    @property
    # @TypeChecking(int)
    def WinMoveHeight(self):
        move_height = self.kwargs.get(GAutoItFieldsMap.winMoveHeight, -1)
        return move_height

    @property
    # @TypeChecking(int)
    def WinMoveWeight(self):
        move_weight = self.kwargs.get(GAutoItFieldsMap.winMoveWeight, -1)
        return move_weight

    @property
    def WinNewTitle(self):
        new_title = self.kwargs.get(GAutoItFieldsMap.winNewTitle, '')
        return new_title

    @property
    # @TypeChecking(int)
    def WinTrans(self):
        trans = self.kwargs.get(GAutoItFieldsMap.winTrans, 255)
        return trans

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinWait(self):
        # print(self.WinTitle)
        return auto.win_wait(title=self.WinTitle, text=self.WinText, timeout=TIMEOUT)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinWaitByHandle(self):
        return auto.win_wait_by_handle(self.WinHandle, timeout=TIMEOUT)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinClose(self):
        # # if self.WinWait():
        return auto.win_close(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinCloseByHandle(self):
        return auto.win_close_by_handle(self.WinHandle)

    @RelatedFields(SendValue=GAutoItFieldsMap.sendValue)
    def Send(self):
        return auto.send(send_text=self.SendValue)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinKill(self):
        if self.WinExists():
            return auto.win_kill(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinKillByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_kill_by_handle(self.WinHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinActivate(self):
        return auto.win_activate(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinActivateByHandle(self):
        return auto.win_activate_by_handle(self.WinHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinWaitActivate(self):
        return auto.win_wait_active(title=self.WinTitle, text=self.WinText, timeout=TIMEOUT)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinWaitActivateByHandle(self):
        return auto.win_wait_active_by_handle(self.WinHandle, timeout=TIMEOUT)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinExists(self):
        return auto.win_exists(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinExistsByHandle(self):
        return auto.win_exists_by_handle(handle=self.WinHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinActive(self):
        return auto.win_active(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinActiveByHandle(self):
        return auto.win_activate_by_handle(handle=self.WinHandle)

    def WinGetCaretPos(self):
        return auto.win_get_caret_pos()

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinGetClassList(self):
        if self.WinExists():
            return auto.win_get_class_list(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinGetClassListByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_get_class_list_by_handle(handle=self.WinHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinGetClientSize(self):
        if self.WinExists():
            return auto.win_get_client_size(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinGetClientSizeByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_get_client_size_by_handle(handle=self.WinHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinGetHandle(self):
        if self.WinExists():
            return auto.win_get_handle(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinGetHandleAsText(self):
        if self.WinExists():
            return auto.win_get_handle_as_text(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinGetPos(self):
        if self.WinExists():
            return auto.win_get_pos(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinGetPosByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_get_pos_by_handle(handle=self.WinHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinGetProcess(self):
        if self.WinExists():
            return auto.win_get_process(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinGetProcessByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_get_process_by_handle(handle=self.WinHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinGetState(self):
        if self.WinExists():
            return auto.win_get_state(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinGetStateByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_get_state_by_handle(handle=self.WinHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinGetText(self):
        if self.WinExists():
            return auto.win_get_text(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinGetTextByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_get_text_by_handle(handle=self.WinHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinGetTitle(self):
        if self.WinExists():
            return auto.win_get_title(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinGetTitleByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_get_title_by_handle(handle=self.WinHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText,
                   MenuItem=GAutoItFieldsMap.menuItem)
    def WinMenuSelectItem(self):
        if self.WinExists():
            return auto.win_menu_select_item(title=self.WinTitle, text=self.WinText, *self.MenuItem)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, MenuItem=GAutoItFieldsMap.menuItem)
    def WinMenuSelectItemByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_menu_select_item_by_handle(handle=self.WinHandle, *self.MenuItem)

    def WinMinimizeAll(self):
        return auto.win_minimize_all()

    def WinMinimizeAllUndo(self):
        return auto.win_minimize_all_undo()

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText,
                   WinMoveX=GAutoItFieldsMap.winMoveX,
                   WinMoveY=GAutoItFieldsMap.winMoveY,
                   WinMoveHeight=GAutoItFieldsMap.winMoveHeight, WinMoveWeight=GAutoItFieldsMap.winMoveWeight)
    def WinMove(self):
        if self.WinExists():
            return auto.win_move(title=self.WinTitle, text=self.WinText, x=self.WinMoveX, y=self.WinMoveY,
                                 height=self.WinMoveHeight, weight=self.WinMoveWeight)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, WinMoveX=GAutoItFieldsMap.winMoveX,
                   WinMoveY=GAutoItFieldsMap.winMoveY,
                   WinMoveHeight=GAutoItFieldsMap.winMoveHeight,
                   WinMoveWeight=GAutoItFieldsMap.winMoveWeight)
    def WinMoveByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_move_by_handle(handle=self.WinHandle, x=self.WinMoveX, y=self.WinMoveY,
                                           height=self.WinMoveHeight, weight=self.WinMoveWeight)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinTopMost(self):
        if self.WinExists():
            return auto.win_set_on_top(title=self.WinTitle, text=self.WinText, flag=1)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinUnTop(self):
        if self.WinExists():
            return auto.win_set_on_top(title=self.WinTitle, text=self.WinText, flag=0)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinTopMostByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_on_top_by_handle(handle=self.WinHandle, flag=1)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinUnTopByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_on_top_by_handle(handle=self.WinHandle, flag=0)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinHide(self):
        if self.WinExists():
            return auto.win_set_state(title=self.WinTitle, text=self.WinText, flag=auto.properties.SW_HIDE)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinHideByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_state_by_handle(handle=self.WinHandle, flag=auto.properties.SW_HIDE)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinShow(self):
        if self.WinExists():
            return auto.win_set_state(title=self.WinTitle, text=self.WinText, flag=auto.properties.SW_SHOW)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinShowByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_state_by_handle(handle=self.WinHandle, flag=auto.properties.SW_SHOW)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinMin(self):
        if self.WinExists():
            return auto.win_set_state(title=self.WinTitle, text=self.WinText, flag=auto.properties.SW_MINIMIZE)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinMinByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_state_by_handle(handle=self.WinHandle, flag=auto.properties.SW_MINIMIZE)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinMax(self):
        if self.WinExists():
            return auto.win_set_state(title=self.WinTitle, text=self.WinText, flag=auto.properties.SW_MAXIMIZE)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinMaxByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_state_by_handle(handle=self.WinHandle, flag=auto.properties.SW_MAXIMIZE)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinRestore(self):
        if self.WinExists():
            return auto.win_set_state(title=self.WinTitle, text=self.WinText, flag=auto.properties.SW_RESTORE)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinRestoreByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_state_by_handle(handle=self.WinHandle, flag=auto.properties.SW_RESTORE)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinNormal(self):
        if self.WinExists():
            return auto.win_set_state(title=self.WinTitle, text=self.WinText, flag=auto.properties.SW_SHOWNORMAL)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinNormalByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_state_by_handle(handle=self.WinHandle, flag=auto.properties.SW_SHOWNORMAL)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinDisable(self):
        if self.WinExists():
            return auto.win_set_state(title=self.WinTitle, text=self.WinText, flag=auto.properties.SW_SHOWNOACTIVATE)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinDisableByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_state_by_handle(handle=self.WinHandle, flag=auto.properties.SW_SHOWNOACTIVATE)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinMinDisable(self):
        if self.WinExists():
            return auto.win_set_state(title=self.WinTitle, text=self.WinText, flag=auto.properties.SW_SHOWMINNOACTIVE)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinMinDisableByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_state_by_handle(handle=self.WinHandle, flag=auto.properties.SW_SHOWMINNOACTIVE)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinNa(self):
        if self.WinExists():
            return auto.win_set_state(title=self.WinTitle, text=self.WinText, flag=auto.properties.SW_SHOWNA)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinNaByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_state_by_handle(handle=self.WinHandle, flag=auto.properties.SW_SHOWNA)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinDefault(self):
        if self.WinExists():
            return auto.win_set_state(title=self.WinTitle, text=self.WinText, flag=auto.properties.SW_SHOWDEFAULT)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinDefaultByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_state_by_handle(handle=self.WinHandle, flag=auto.properties.SW_SHOWDEFAULT)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText,
                   WinNewTitle=GAutoItFieldsMap.winNewTitle)
    def WinSetTile(self):
        if self.WinExists():
            return auto.win_set_title(title=self.WinTitle, text=self.WinText, new_title=self.WinNewTitle)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, WinNewTitle=GAutoItFieldsMap.winNewTitle)
    def WinSetTitleByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_title_by_handle(handle=self.WinHandle, new_title=self.WinNewTitle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText,
                   WinTrans=GAutoItFieldsMap.winTrans)
    def WinSetTrans(self):
        if self.WinExists():
            return auto.win_set_trans(title=self.WinTitle, text=self.WinText, trans=self.WinTrans)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, WinTrans=GAutoItFieldsMap.winTrans)
    def WinSetTransByHandle(self):
        if self.WinExistsByHandle():
            return auto.win_set_trans_by_handle(handle=self.WinHandle, trans=self.WinTrans)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinWaitClose(self):
        return auto.win_wait_close(title=self.WinTitle, text=self.WinText, timeout=TIMEOUT)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinWaitCloseByHandle(self):
        return auto.win_wait_close_by_handle(handle=self.WinHandle, timeout=TIMEOUT)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def WinWaitNotActive(self):
        return auto.win_wait_not_active(title=self.WinTitle, text=self.WinText, timeout=TIMEOUT)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def WinWaitNotActiveByHandle(self):
        return auto.win_wait_not_active_by_handle(handle=self.WinHandle, timeout=TIMEOUT)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   WinText=GAutoItFieldsMap.winText)
    def CtlLeftClick(self):
        # print(self.WinTitle)
        if self.WinExists():
            return auto.control_click(title=self.WinTitle, text=self.WinText, control=self.CtlClassNameNN,
                                      button='left')

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle)
    def CtlLeftClickByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_click_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle, button='left')

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   WinText=GAutoItFieldsMap.winText)
    def CtlDoubleclick(self):
        if self.WinExists():
            return auto.control_click(title=self.WinTitle, text=self.WinText, control=self.CtlClassNameNN,
                                      button='left', click='2')

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle)
    def CtlDoubleClickByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_click_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle, button='left', click='2')

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   WinText=GAutoItFieldsMap.winText)
    def CtlRightClick(self):
        if self.WinExists():
            return auto.control_click(title=self.WinTitle, text=self.WinText, contrl=self.CtlClassNameNN,
                                      button='right')

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle)
    def CtlRightClickByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_click_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle, button='right')

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   WinText=GAutoItFieldsMap.winText)
    def CtlRightDoubleClick(self):
        if self.WinExists():
            return auto.control_click(title=self.WinTitle, text=self.WinText, contrl=self.CtlClassNameNN,
                                      button='right', click='2')

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle)
    def CtlRightDoubleClickByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_click_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle, button='right', click='2')

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   CtlText=GAutoItFieldsMap.ctlText, SendValue=GAutoItFieldsMap.sendValue)
    def CtlSend(self):
        if self.WinExists():
            return auto.control_send(title=self.WinTitle, text=self.WinText, control=self.CtlClassNameNN,
                                     send_text=self.CtlSendValue, mode=0)
        # else:
        #     return None

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle,
                   SendValue=GAutoItFieldsMap.sendValue)
    def CtlSendByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_send_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle, send_text=self.CtlSendValue,
                                               mode=0)
        # else:
        #     return None

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   CtlText=GAutoItFieldsMap.ctlText, WinText=GAutoItFieldsMap.winText)
    def CtlSetText(self):
        if self.WinExists():
            return auto.control_set_text(title=self.WinTitle, text=self.WinText, control=self.CtlClassNameNN,
                                         control_text=self.CtlText)
        # else:
        #     return None

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle,
                   CtlText=GAutoItFieldsMap.ctlText)
    def CtlSetTextByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_set_text_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle,
                                                   control_text=self.CtlText)
        # else:
        #     return None

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   Command=GAutoItFieldsMap.command, CommandExtra=GAutoItFieldsMap.commandExtra,
                   WinText=GAutoItFieldsMap.winText)
    def CtlCommand(self):
        if self.WinExists():
            return auto.control_command(title=self.WinTitle, text=self.WinText, control=self.CtlClassNameNN,
                                        command=self.Command, extra=self.CommandExtra)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle,
                   Command=GAutoItFieldsMap.command,
                   CommandExtra=GAutoItFieldsMap.commandExtra)
    def CtlCommandByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_command_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle, command=self.Command,
                                                  extra=self.CommandExtra)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText,
                   CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   ListViewCommand=GAutoItFieldsMap.listViewCommand,
                   ListViewExtra1=GAutoItFieldsMap.listViewExtra1, ListViewExtra2=GAutoItFieldsMap.listViewExtra2)
    def CtlListView(self):
        if self.WinExists():
            return auto.control_list_view(title=self.WinTitle, text=self.WinText, command=self.ListViewCommand,
                                          control=self.CtlClassNameNN,
                                          extra1=self.ListViewExtra1, extra2=self.ListViewExtra2)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle,
                   ListViewCommand=GAutoItFieldsMap.listViewCommand,
                   ListViewExtra1=GAutoItFieldsMap.listViewExtra1, ListViewExtra2=GAutoItFieldsMap.listViewExtra2)
    def CtlListViewByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_list_view_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle,
                                                    command=self.ListViewCommand,
                                                    extra1=self.ListViewExtra1, extra2=self.ListViewExtra2)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   WinText=GAutoItFieldsMap.winText)
    def CtlDisable(self):
        if self.WinExists():
            return auto.control_disable(title=self.WinTitle, control=self.CtlClassNameNN, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle)
    def CtlDisableByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_disable_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   WinText=GAutoItFieldsMap.winText)
    def CtlEnable(self):
        if self.WinExists():
            return auto.control_enable(title=self.WinTitle, control=self.CtlClassNameNN, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle)
    def CtlEnableByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_enable_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   WinText=GAutoItFieldsMap.winText)
    def CtlFocus(self):
        if self.WinExists():
            return auto.control_focus(title=self.WinTitle, control=self.CtlClassNameNN, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle)
    def CtlFocusByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_focus_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText)
    def CtlGetFocus(self):
        if self.WinExists():
            return auto.control_get_focus(title=self.WinTitle, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle)
    def CtlGetFocusByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_get_focus_by_handle(hwnd=self.WinHandle)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN)
    def CtlGetHandle(self):
        if self.WinExistsByHandle():
            return auto.control_get_handle(hwnd=self.WinHandle, control=self.CtlClassNameNN)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   WinText=GAutoItFieldsMap.winText)
    def CtlGetHandleAsText(self):
        if self.WinExists():
            return auto.control_get_handle_as_text(title=self.WinTitle, control=self.CtlClassNameNN, text=self.WinText)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   WinText=GAutoItFieldsMap.winText)
    def CtlGetPos(self):
        if self.WinExists():
            return auto.control_get_pos(title=self.WinTitle, control=self.CtlClassNameNN, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle)
    def CtlGetPosByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_get_pos_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   WinText=GAutoItFieldsMap.winText)
    def CtlGetText(self):
        if self.WinExists():
            return auto.control_get_text(title=self.WinTitle, control=self.CtlClassNameNN, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle)
    def CtlGetTextByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_get_text_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   WinText=GAutoItFieldsMap.winText)
    def CtlHide(self):
        if self.WinExists():
            return auto.control_hide(title=self.WinTitle, control=self.CtlClassNameNN, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle)
    def CtlHideByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_hide_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, WinText=GAutoItFieldsMap.winText,
                   CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN, CtlMoveX=GAutoItFieldsMap.ctlMoveX,
                   CtlMoveY=GAutoItFieldsMap.ctlMoveY, CtlMoveWidth=GAutoItFieldsMap.ctlMoveWidth,
                   CtlMoveHeight=GAutoItFieldsMap.ctlMoveHeight)
    def CtlMove(self):
        if self.WinExists():
            return auto.control_move(title=self.WinTitle, text=self.WinText, control=self.CtlClassNameNN,
                                     x=self.CtlMoveX,
                                     y=self.CtlMoveY, width=self.CtlMoveWidth, height=self.CtlMoveHeight)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle,
                   CtlMoveX=GAutoItFieldsMap.ctlMoveX,
                   CtlMoveY=GAutoItFieldsMap.ctlMoveY,
                   CtlMoveWidth=GAutoItFieldsMap.ctlMoveWidth, CtlMoveHeight=GAutoItFieldsMap.ctlMoveHeight)
    def CtlMoveByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_move_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle, x=self.CtlMoveX,
                                               y=self.CtlMoveY,
                                               width=self.CtlMoveWidth, height=self.CtlMoveHeight)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   WinText=GAutoItFieldsMap.winText)
    def CtlShow(self):
        if self.WinExists():
            return auto.control_show(title=self.WinTitle, control=self.CtlClassNameNN, text=self.WinText)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle)
    def CtlShowByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_show_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle)

    @RelatedFields(WinTitle=GAutoItFieldsMap.winTitle, CtlClassNameNN=GAutoItFieldsMap.ctlClassNameNN,
                   TreeViewCommand=GAutoItFieldsMap.treeViewCommand,
                   WinText=GAutoItFieldsMap.winText, TreeViewExtra1=GAutoItFieldsMap.treeViewExtra1,
                   TreeViewExtra2=GAutoItFieldsMap.treeViewExtra2)
    def CtlTreeView(self):
        if self.WinExists():
            return auto.control_tree_view(title=self.WinTitle, control=self.CtlClassNameNN,
                                          command=self.TreeViewCommand,
                                          text=self.WinText, extras1=self.TreeViewExtra1, extras2=self.TreeViewExtra2)

    @RelatedFields(WinHandle=GAutoItFieldsMap.winHandle, CtlHandle=GAutoItFieldsMap.ctlHandle,
                   TreeViewCommand=GAutoItFieldsMap.treeViewCommand,
                   TreeViewExtra1=GAutoItFieldsMap.treeViewExtra1, TreeViewExtra2=GAutoItFieldsMap.treeViewExtra2)
    def CtlTreeViewByHandle(self):
        if self.WinExistsByHandle():
            return auto.control_tree_view_by_handle(hwnd=self.WinHandle, h_ctrl=self.CtlHandle,
                                                    command=self.TreeViewCommand,
                                                    extra1=self.TreeViewExtra1, extra2=self.TreeViewExtra2)


def execute(operations: (list, dict)):
    if isinstance(operations, list):
        for operation in operations:
            g_control_obj = GAutoIt(operation_detail=operation)
            g_control_obj.start()
    else:
        g_control_obj = GAutoIt(operation_detail=operations)
        g_control_obj.start()


if __name__ == '__main__':
    demo = {
        "SerialNum": "",
        "IsUIA": "False",
        "Title": "",
        "Control": "",
        "OpenApp": "",
        "SearchFromControl": "",
        "SearchDepth": '',
        "SearchInterval": "0.5",
        "Name": "name",
        "ClassName": "",
        "SubName": "",
        "AutomationId": "",
        "FoundIndex": '',
        "ControlType": "",
        "Condiment": "",
        "ChildrenIndex": "",
        "Operation": "HotKey",
        "Value": "",
        "CtlHandle": 1
    }
    g = GAutoIt(operation_detail=demo)
    print(g.CtlTreeViewByHandle())



import autoit


class GAutoMap:
    MouseClick = 'MouseClick'
    MouseMove = 'MouseMove'
    MouseDrag = 'MouseDrag'
    MouseWheel = 'MouseWheel'


class GMouseControl(object):
    """
    AutoIt鼠标相关操作
    """

    def __init__(self):
        """
        Constructor
        """

    @staticmethod
    def Click(title, text, x, y, button="main", clicks=1):
        """
        :description 执行鼠标点击操作
        """
        pos = autoit.win_get_pos(title, text=text)
        autoit.mouse_click(button, x + pos[0], y + pos[1], clicks=clicks)

    @staticmethod
    def Move(title, text, x, y):
        """
        :description 移动鼠标指针
        """
        pos = autoit.win_get_pos(title, text=text)
        autoit.mouse_move(x + pos[0], y + pos[1])

    @staticmethod
    def Drag(title, text, x1, y1, x2, y2):
        """
        :description 执行鼠标拖拽操作
        """
        pos = autoit.win_get_pos(title, text=text)
        autoit.mouse_click_drag(x1 + pos[0], y1 + pos[1], x2 + pos[0], y2 + pos[1])

    @staticmethod
    def Wheel(direction="up"):
        """
        :description 产生向上或向下滚动鼠标滚轮事件.仅支持NT/2000/XP及更高.
        """
        autoit.mouse_wheel(direction)


class GProcessControl(object):
    """
    AutoIt进程相关操作
    """

    def __init__(self, processName):
        """
        Constructor
        """
        self.processName = processName

    def Close(self):
        """
        :description 终止某个进程
        :return 1:成功; 0:失败.
        """
        return autoit.process_close(self.processName)

    def Exists(self):
        """
        :description 检查指定进程是否存在
        :return PID:成功; 0:进程不存在.
        """
        return autoit.process_exists(self.processName)


class GWinControl(object):
    """
    AutoIt窗口相关操作
    """

    def __init__(self, title, text=''):
        """
        Constructor
        """
        self.title = title
        self.text = text

    @staticmethod
    def open_exe(title, exe_path, timeout=30):
        """
        打开软件并等待加载完成，窗口最大化
        :return:
        """
        try:
            autoit.run(exe_path)
            autoit.win_wait_active(title, timeout)
            autoit.win_set_state(title, flag=autoit.autoit.Properties.SW_MAXIMIZE)
        except Exception as e:
            print(e)
            print('软件打开失败')
            exit(1)

    def Activate(self):
        """
        :description 激活指定的窗口(设置焦点到该窗口,使其成为活动窗口).
        :return PID:窗口存在; 0:窗口不存在.
        """
        return autoit.win_activate(self.title, text=self.text)

    def Close(self):
        """
        :description 关闭指定窗口.
        :return 1:成功; 0:窗口不存在.
        """
        return autoit.win_close(self.title, text=self.text)

    def Exists(self):
        """
        :description 检查指定的窗口是否存在.
        :return 1:窗口存在; 0:窗口不存在.
        """
        return autoit.win_exists(self.title, text=self.text)

    def GetPos(self):
        """
        :description 获取指定窗口的坐标位置和大小等属性.
        :return Returns left, top, right, bottom    (x1,y1,x2,y2)
        """
        return autoit.win_get_pos(self.title, text=self.text)

    def GetProcess(self):
        """
        :description 获取指定窗口关联的进程ID(PID).
        :return PID:成功, -1:失败.
        """
        return autoit.win_get_process(self.title, text=self.text)

    def GetText(self, buf_size=256):
        """
        :description 获取指定窗口中的文本.
        :return  指定窗口里包含的文本:成功; 0:失败(没有匹配的窗口).
        """
        return autoit.win_get_text(self.title, buf_size, text=self.text)

    def Kill(self):
        """
        :description 强行关闭指定窗口.
        :return  1:无论成功失败.
        """
        return autoit.win_kill(self.title, text=self.text)

    def Move(self, x, y, width, height):
        """
        :description 移动指定的窗口或调整窗口的大小.
        :return PID:成功, 0:失败(窗口不存在).
        """
        return autoit.win_move(self.title, x, y, width, height, text=self.text)

    def SetState(self, flag):
        """
        :description 显示,隐藏,最小化,最大化或还原一个窗口.
        :param flag: The "show" flag of the executed program:
                        = 显示
                        = 最小化/隐藏
                        = 最大化
                        = 还原
        :return 1:成功, 0:失败(窗口不存在).
        """
        return autoit.win_set_state(self.title, flag, text=self.text)

    def Wait(self, timeout=5):
        """
        :description 暂停脚本的执行直至指定窗口存在(出现)为止.
        timeout 单位为秒.
        :return PID:成功, 0:失败(超时).
        """
        return autoit.win_wait(self.title, timeout, text=self.text)

    def WaitActive(self, timeout=5):
        """
        :description 暂停脚本的执行直至指定窗口被激活(成为活动状态)为止.
        timeout 单位为秒.
        :return PID:成功, 0:失败(超时).
        """
        return autoit.win_wait_active(self.title, timeout, text=self.text)

    def WaitClose(self, timeout=5):
        """
        :description 暂停脚本的执行直至所指定窗口不再存在为止.
        timeout 单位为秒.
        :return 1:成功, 0:失败(超时).
        """
        return autoit.win_wait_close(self.title, timeout, text=self.text)

    def WaitNotActive(self, timeout=5):
        """
        :description 暂停脚本的执行直至指定窗口不是激活状态为止.
        timeout 单位为秒.
        :return 1:成功, 0:失败(超时).
        """
        return autoit.win_wait_not_active(self.title, timeout, text=self.text)

    def ControlClick(self, control, button="main", clicks=1):
        """
        :description 向指定控件发送鼠标点击命令.
        """
        autoit.control_click(self.title, control, text=self.text, button=button, clicks=clicks)

    def ControlCommand(self, control, command, extra="", buf_size=256):
        """
        :param buf_size:
        :param command:
        :param control:
        :description 向指定控件发送命令.
        :param command, extra:
        :return
                    "IsVisible", ""                    1:可见; 0:不可见
                    "IsEnabled", ""                    1:可用; 0:禁用
                    "ShowDropDown", ""                弹出/下拉 组合框(ComboBox)的列表.
                    "HideDropDown", ""                收回/隐藏 组合框(ComboBox)的列表.
                    "AddString", "string"                在 ListBox 或 ComboBox 的编辑框后面附加指定字符串.
                    "DelString", 出现次序                                删除在 ListBox 或 ComboBox 的编辑框中指定的字符串(从0开始).
                    "FindString", "string"                返回在 ListBox 或 ComboBox 的编辑框中与指定字符串匹配项目的出现次序(从0开始).
                    "SetCurrentSelection", 出现次序            通过指定出现次序(从0开始)把 ListBox 或 ComboBox 的当前选择项设为指定的项目.
                    "SelectString","string"            通过指定字符串把 ListBox 或 ComboBox 的当前选择项设为匹配字符串的项目.
                    "IsChecked", ""                    若目标按钮(复选框/单选框)被选中则返回值为1,否则为0.
                    "Check", ""                        使目标按钮(复选框/单选框)变为选中状态.
                    "UnCheck", ""                        使目标按钮(复选框/单选框)变为非选中状态.
                    "GetCurrentLine", ""                    返回在目标编辑框中插入符(caret,光标)的所在行号.
                    "GetCurrentCol", ""                    返回在目标编辑框中插入符(caret,光标)的所在列号.
                    "GetCurrentSelection", ""                返回 ListBox 或 ComboBox 控件当前选中的项目名.
                    "GetLineCount", ""                        返回目标编辑框中的总行数.
                    "GetLine", 行号                                        返回目标编辑框中指定行的文本内容.
                    "GetSelected", ""                    返回目标编辑框中的(用户用鼠标或其它方式)选定的文本.
                    "EditPaste", 'string'                在目标编辑框中插入符(caret)所在位置后插入指定字符串.
                    "CurrentTab", ""                    返回在 SysTabControl32 控件中当前显示的标签编号(从1开始).
                    "TabRight", ""                        使 SysTabControl32 控件切换到(右边的)下一个标签.
                    "TabLeft", ""                        使 SysTabControl32 控件切换到(左边的)下一个标签.
                    "SendCommandID", 命令 ID            模拟 WM_COMMAND 消息. 通常用于 ToolbarWindow32 控件 - 使用Au3Info的工具栏标签得到命令ID.
        """
        return autoit.control_command(self.title, control, command, buf_size, text=self.text, extra=extra)

    def ControlListView(self, control, command, extra1, extra2="", buf_size=256):
        """
        :param buf_size:
        :param extra1:
        :param command:
        :param control:
        :description 向指定的 ListView32 控件发送命令.
        :param command, extra1, extra2:
        :return
                    "DeSelect", 从[, 到]                        取消选定从"从"开始直到"到"的一个或多个项目.
                    "FindItem", "搜索字符串" [, 子项目]            返回与给定字符串匹配的项目的位置.若未找到指定字符串则返回值为 -1.
                    "GetItemCount"                                返回列表中项目的数量.
                    "GetSelected" [, 选项]                        返回当前选中项目的位置.若 选项=0(默认)则只返回选中的第一个项目;
                                                                若 选项=1 则返回由竖线"|"作为分隔符的所有选中项目,
                                                                例如:"0|3|4|10".若没有选中任何项目则返回一个空字符串"".
                    "GetSelectedCount"                            返回选中项目的数量.
                    "GetSubItemCount"                            返回子项目的数量.
                    "GetText", 项目, 子项目                                            返回指定项目/子项目的文本.
                    "IsSelected", 项目                                                    若指定项目被选中则返回值为1,否则返回值为0.
                    "Select", 从[, 到]                            选中一个或多个项目(请参考第一个命令).
                    "SelectAll"                                    选中所有项目.
                    "SelectClear"                                取消所有项目的选中状态.
                    "SelectInvert"                                切换当前的选中状态.
                    "ViewChange", "视图"                        切换当前的视图.可用的视图包括"list"(列表),"details"(详细信息),
                    "small icons"(小图标),
                    "large icons"(大图标).
        """
        return autoit.control_list_view(self.title, control, command, buf_size, text=self.text, extra1=extra1,
                                        extra2=extra2)

    def ControlDisable(self, control):
        """
        :description 禁用或使某控件变成灰色不可用状态.
        :return 1:成功; 0:失败.
        """
        return autoit.control_disable(self.title, control, text=self.text)

    def ControlEnable(self, control):
        """
        :description 使灰色按钮/控件变为"可用"状态.
        :return 1:成功; 0:失败.
        """
        return autoit.control_enable(self.title, control, text=self.text)

    def ControlFocus(self, control):
        """
        :description 设置输入焦点到指定窗口的某个控件上.
        :return 1:成功; 0:失败.
        """
        return autoit.control_focus(self.title, control, text=self.text)

    def ControlGetText(self, control):
        """
        :description 获取指定控件上的文本.
        :return 文本内容:成功; 空:失败.
        """
        return autoit.control_get_text(self.title, control, text=self.text)

    def ControlSend(self, control, send_text, mode=0):
        """
        :param send_text:
        :param control:
        :description 向指定的控件发送字符串.
        :param mode: 0:按特殊字符含义发送(默认); 1:原样发送.
        :return 1:成功; 0:失败(窗口/控件未找到).
        """
        return autoit.control_send(self.title, control, send_text, mode, text=self.text)

    def ControlSetText(self, control, control_text):
        """
        :description 修改指定控件的文本.
        :return 1:成功; 0:失败(窗口/控件未找到).
        """
        return autoit.control_set_text(self.title, control, control_text, text=self.text)

    def ControlTreeView(self, control, command, extra, buf_size=256):
        """
        :param extra:
        :param buf_size:
        :param command:
        :param control:
        :description 发送一个命令到 TreeView32 控件.(子节点不好用*)
        :param command, extra
        :return
                "Check", "项目"                                             选中一个项目 (如果项目支持选中,这里指项目带有选择框).
                "Collapse", "项目"                                          折叠一个项目节点,使它隐藏它的子项目.
                "Exists", "项目"         *都返回1*                      如果项目存在返回 1,否则返回 0.
                "Expand", "项目"                                           展开一个项目节点,使它显示它的子项目.
                "GetItemCount", "项目"                                 返回所选项目的子项目数量.
                "GetSelected" [, 使用索引]                             返回当前所选项目的文本参考信息(如果使用索引设置为1将会返回所选项目索引位置).
                "GetText", "项目"                                          返回项目文本.
                "IsChecked"                                                  返回项目选中状态. 1:被选中, 0:未被选中, -1:没要选择框.
                "Select", "项目"                                             选择一个项目.
                "Uncheck", "项目"                                         取消项目选中状态 (如果项目支持选中,这里指项目带有选择框).
        """
        return autoit.control_tree_view(self.title, control, command, text=self.text, buf_size=buf_size, extra=extra)

    def StatusBarGetText(self, part=1, buf_size=256):
        """
        :description 获取标准状态栏控件的文本.
        :return 文本内容:成功; 空字符串:失败(无法读取文本).
        """
        return autoit.statusbar_get_text(self.title, self.text, part, buf_size)

    @staticmethod
    def Send(send_text):
        return autoit.send(send_text=send_text)


if __name__ == '__main__':

    pass


import autoit
import time
import pyperclip as pyperclip
from PIL import ImageGrab


class GPyAutoIt(object):
    @staticmethod
    def open_exe(title, exe_path, timeout=30):
        """
        打开软件并等待加载完成，窗口最大化
        :return:
        """
        try:
            autoit.run(exe_path)
            autoit.win_wait_active(title, timeout)
            autoit.win_set_state(title, flag=autoit.autoit.Properties.SW_MAXIMIZE)
        except Exception as e:
            print(e)
            print('软件打开失败')
            exit(1)

    @staticmethod
    def send_keys(title, control, text):
        """
        指定输入框输入文本
        :param title: 主窗口标题
        :param control:  对应的控件的 Class name NN
        :param text: 要输入的文本
        :return:
        """
        autoit.control_click(title, control)
        time.sleep(0.1)

        # 复制
        pyperclip.copy(text)
        # ctrl + v
        autoit.send('{CTRL DOWN}')
        autoit.send('{v down}')
        autoit.send('{v up}')
        autoit.send('{CTRL UP}')

    @staticmethod
    def control_click(title, control):
        """
        点击该打开软件的某个控件
        :param title: 主窗口标题
        :param control: 对应的控件的 Class name NN
        :return: 返回点击控件后的界面的text control对
        """
        autoit.control_click(title, control)
        if title == 'xxxxxx':
            time.sleep(2)

        return GPyAutoIt.get_dic(title)

    @staticmethod
    def control_click_no_wait(title, control):
        """
        点击该打开软件的某个控件
        :param title: 主窗口标题
        :param control: 对应的控件的 Class name NN
        :return: 返回点击控件后的界面的text control对
        """
        autoit.control_click(title, control)

    @staticmethod
    def close_notice_window(title='提示', control=None, timeout=10):
        """
        点击该打开软件的提示窗口的某个控件
        :param title: 提示窗口标题
        :param control: 对应的控件的 Class name NN
        :param timeout: 超时时间
        :return:
        """
        def close_inform():
            autoit.send('{TAB}')
            autoit.send('{TAB}')
            autoit.send('{SPACE}')
            autoit.send('{TAB}')
            autoit.send('{SPACE}')

            autoit.send('{TAB}')
            autoit.send('{TAB}')
            autoit.send('{SPACE}')
            autoit.send('{TAB}')
            autoit.send('{SPACE}')

            autoit.send('{SPACE}')
            autoit.send('{SPACE}')

        try:
            if title == '提示':
                autoit.win_wait_active(title, timeout)
                autoit.control_click(title, control)
            elif title == '告知书':
                autoit.win_wait(title, timeout)
                autoit.mouse_click(x=800, y=400)  # 点击告知书提示框使其被选中
                close_inform()

                autoit.send('{SPACE}')
                autoit.send('{SPACE}')

        except Exception as e:
            print('{}窗口关闭失败, msg : {}'.format(title, str(e)))

    @staticmethod
    def ca_login(title='FormAccountPassword', psw_input_control=None, login_control=None, timeout=10):
        """
        增值税 一般纳税人
        进入该表时要用CA密码验证
        :param title:
        :param psw_input_control:
        :param login_control:
        :param timeout:
        :return:
        """
        autoit.win_wait_active(title, timeout)
        GPyAutoIt.send_keys(title, psw_input_control, 'xxxxxx')
        GPyAutoIt.control_click(title, login_control)

    @staticmethod
    def get_username(title, username_list):
        """
        获得账号名和对应的control组成的dict
        :param title:
        :param username_list: 账号（公司全称）构成的list
        :return:dict， key为公司全称， value为对应的control
        """
        dic = dict()

        for key, value in GPyAutoIt.get_dic(title).items():
            if key in username_list:
                dic[key] = value

        return dic

    @staticmethod
    def get_dic(title):
        # :param control ：要获取的控件的Class
        """
        :param title:
        :return: 由text和对应的control组成的字典， key：text， value：control
        """
        static_control = 'WindowsForms10.STATIC.app.0.33c0d9d'
        button_control = 'WindowsForms10.BUTTON.app.0.33c0d9d'
        edit_control = 'WindowsForms10.EDIT.app.0.33c0d9d'

        def get_d(control_str, n=500):
            d = dict()
            for s in (control_str + str(i) for i in range(1, n)):
                try:
                    key = autoit.control_get_text(title, s)
                    if key == '':
                        continue
                    elif key in d.keys():
                        if isinstance(d[key], list):
                            d[key] = d[key].append(s)
                        else:
                            d[key] = [d[key], s]
                    else:
                        d[key] = s
                except autoit.autoit.AutoItError:
                    continue
            return d

        return dict(get_d(static_control), **get_d(button_control), **get_d(edit_control))

    @staticmethod
    def get_text_control(title):
        """
        获取该界面中的text和其对应的control
        :param title: 窗口标题
        :return: 由text和对应的control组成的字典， key：text， value：control
        """
        return GPyAutoIt.get_dic(title)

    @staticmethod
    def switch_account(title, username, username_list):
        """
        切换账号
        :param title: 主窗口标题
        :param username_list: 公司全称列表
        :param username: 要切换的公司全称
        :return:
        """
        dic = GPyAutoIt.get_username(title, username_list)
        return GPyAutoIt.control_click(title, dic[username])

    @staticmethod
    def get_check_result(title, timeout=2):
        """
        获取审核结果
        :param title:
        :param timeout:
        :return:返回审核结果
        """
        autoit.win_wait_active(title, timeout)
        return autoit.win_get_text(title)

    @staticmethod
    def exit_table(title, control='WindowsForms10.BUTTON.app.0.33c0d9d1', timeout=5):
        """
        返回上一级，返回税表的上一级(提示要保存表格数据时，默认不保存)
        :param title:
        :param control:
        :param timeout:
        :return:
        """
        GPyAutoIt.control_click(title, control)
        try:
            autoit.win_wait_active('提示', timeout)
            GPyAutoIt.control_click('提示', 'Button2')
        except autoit.autoit.AutoItError:
            pass

    @staticmethod
    def get_text_by_control(title, control):
        """
        根据给定的control 获取对应的text
        :param title:
        :param control:
        :return: ctrl对应的text
        """
        return autoit.control_get_text(title, control)

    @staticmethod
    def screen_capture(title, control, text=''):
        """
        对相应的控件截图
        :param title:
        :param control:
        :param text:
        :return:
        """
        left, top, right, bottom = autoit.control_get_pos(title, control, text)
        img = ImageGrab.grab((left, top, right, bottom))
        img.show()


if __name__ == '__main__':

    pass


# -*- coding: utf-8 -*-
import os
from setuptools import setup, find_packages

path = os.path.abspath(os.path.dirname(__file__))
try:
    with open(os.path.join(path, 'README.MD'), encoding='utf-8') as f:
        long_description = f.read()
except Exception as e:
    long_description = "The configuration file is automatically executed"
package = find_packages(exclude=['Download-bank-statement'], include=['main'])

setup(
    name="executor1",
    version="1.0.0",
    keywords=("uiautomation", "autoit", "python"),
    description="The configuration file is automatically executed",
    long_description=long_description,
    long_description_content_type='text/markdown',
    python_requires=">=3.6.0",
    author="rejii",
    license='MIT',
    include_package_data=True,
    author_email="ljy@bczx365.com",
    packages=package,
    install_requires=['pyautoit', 'uiautomation'],
    url="http://gitlab.bczx365.com/rpa/UIA_Executor",
)
print(package)




import uiautomation


def control_dic(name, depth, control_type):
    """
    :param name: inspect name 列的值
    :param depth: 层数
    :param control_type:  control 类型 如 TextControl EditControl ButtonControl
    :return: obj
    """
    obj = None
    if control_type == 'TextControl':
        obj = uiautomation.TextControl(Depth=depth, Name=name)
    elif control_type == 'EditControl':
        obj = uiautomation.EditControl(Depth=depth, Name=name)
    elif control_type == 'ButtonControl':
        obj = uiautomation.ButtonControl(Depth=depth, Name=name)
    return obj


def get_next_control(name, depth, control_type):
    """
    获取当前control对象的下一个control
    :param name: inspect name 列的值
    :param depth: 层数
    :param control_type:  control 类型 如 TextControl EditControl ButtonControl
    :return: obj
    """
    obj = control_dic(name, depth, control_type)
    if not obj:
        raise Exception('control_type not exist')
    parent = obj.GetParentControl().GetChildren()
    for i, v in enumerate(parent):
        if v.Name == name:
            return parent[i+1]


def get_same_level_control_by_offset(name, depth, control_type, offset):
    """
    在同一层级中，通过name唯一的control得到相邻offset步的control对象
    offset: 如：1，2，-1，-3  | 1代表下边一位， -3代表上边3位
    正数代表 目标control = name + offset
    负数代表 目标control = name - offset
    :param name: inspect name 列的值
    :param depth: 层数
    :param control_type:  control 类型 如 TextControl EditControl ButtonControl
    :param offset: 相邻间隔步数
    :return: obj
    """
    obj = control_dic(name, depth, control_type)
    if not obj:
        raise Exception('control_type not exist')
    parent = obj.GetParentControl().GetChildren()
    for i, v in enumerate(parent):
        if v.Name == name:
            return parent[i+offset]


def get_value_from_edit_control(obj):
    """
    获取当前Control的value值，只支持EditControl
    """
    return obj.GetValuePattern().Value


def pattern_set_by_exit_control(obj, val):
    """
    不丢字符设置值，只支持EditControl
    特点: 1.不丢字符  普调SetValue() 会丢字符
         2.直接覆盖，不需要清空编辑框
         3.目前只支持EditControl,其他类型control无次方法
    :param obj: EditControl obj
    :param val: 要设置的值
    """
    obj.GetValuePattern().SetValue(str(val))


def get_legacy_iaccessible_value(obj):
    """
    获取当前Control的LegacyIAccessible value值
    有时候,inspect只有LegacyIAccessible有值, 当普调Value无值时用这个获取
    """
    return obj.GetLegacyIAccessiblePattern().Value



#!/usr/bin/python
# -*- coding: UTF-8 -*-


from setuptools import setup
# from distutils.core import setup
from Cython.Build import cythonize

setup(
    name='py2c',
    # 打包文件的名称，默认与py2c.py在同一路径下
    ext_modules=cythonize(r'D:\BCZX\RPA\self\GExecutor\main\test.py')
)